/* global firstDay, holidaySchemes, loggedInUser, loggedUserIsHR, teams, userIsHR, vacationTypes */

AJS.$(document).ready(function() {
    var MONDAY_INDEX = 1;
    var TUESDAY_INDEX = 2;
    var WEDNESDAY_INDEX = 3;
    var THURSDAY_INDEX = 4;
    var FRIDAY_INDEX = 5;
    var DATE_TIME_PARSE_FORMAT = 'yyyy-MM-DD HH:mm';
    var DATE_DISPLAY_FORMAT = 'DD-MM-YYYY';
    var daysOfWork = [MONDAY_INDEX, TUESDAY_INDEX, WEDNESDAY_INDEX, THURSDAY_INDEX, FRIDAY_INDEX];

    var teamMembers = [];
    var supervisors = [];
    var vacations = [];
    var issueKeyFromUrl = vmUtils.getUrlQueryParams().issueKey;

    if (teamIdIsNotPresentInUrl() && teams.length > 0) {
        updateTeamIdInUrl(teams[0].id);
    } else if (teamIdInUrlDoesNotMatchAnyAvailableTeam() && teams.length > 0) {
        updateTeamIdInUrl(teams[0].id);
    }

    if (teams.length > 1) {
        initializeTeamSelection();
    } else {
        hideTeamSelection();

        if (teams.length === 1) {
            displayTeamNameInHeader(teams[0]);
        }
    }

    var teamCalendar = VM$('#statistics-team-calendar').fullCalendar({
        header: {
            center: 'prevYear,prev title next,nextYear',
            right: 'today'
        },
        firstDay: firstDay,
        aspectRatio: 1.7,
        businessHours: {dow: daysOfWork},
        displayEventTime: false,
        viewRender: function () {
            setTimeout(displayYearlyHolidays);
        },
        eventRender: onCalendarEventRender
    });

    fetchAndDisplayTeamStatistics(vmUtils.getUrlQueryParams().teamId);

    function teamIdIsNotPresentInUrl() {
        var urlQueryParams = vmUtils.getUrlQueryParams();
        return !urlQueryParams['teamId'];
    }

    function updateTeamIdInUrl(teamId) {
        if (window.location.search) {
            window.history.replaceState(null, null, getCurrentUrlWithTeamId(teamId));
        } else {
            window.history.replaceState(null, null, '?teamId=' + encodeURIComponent(teamId));
        }
    }

    function teamIdInUrlDoesNotMatchAnyAvailableTeam() {
        var teamIdFromUrl = parseInt(vmUtils.getUrlQueryParams().teamId, 10);
        for (var i = 0; i < teams.length; i++) {
            if (teams[i].id === teamIdFromUrl) {
                return false;
            }
        }

        return true;
    }

    function getCurrentUrlWithTeamId(newTeamId) {
        var urlQueryParams = vmUtils.getUrlQueryParams();
        urlQueryParams.teamId = newTeamId;

        var newParams = [];
        for (var paramName in urlQueryParams) {
            newParams.push(encodeURIComponent(paramName) + '=' + encodeURIComponent(urlQueryParams[paramName]));
        }

        return window.location.origin + window.location.pathname + '?' + newParams.join('&');
    }

    function initializeTeamSelection() {
        teams.forEach(function (team) {
            AJS.$('#statistics-team-select-team').append(AJS.$('<option>').text(team.name).val(team.id));
        });

        var teamId = vmUtils.getUrlQueryParams().teamId;
        AJS.$('#statistics-team-select-team option[value=' + teamId + ']').prop('selected', 'selected');
        AJS.$('#statistics-team-select-team')
            .auiSelect2()
            .change(onTeamChanged);
    }

    function onTeamChanged(event) {
        var teamId = event.added.element[0].value;
        updateTeamIdInUrl(teamId);
        fetchAndDisplayTeamStatistics(teamId);
    }

    function fetchAndDisplayTeamStatistics(teamId) {
        var fetchVacations = VM$.ajax({
            type : 'GET',
            url : AJS.contextPath() + '/rest/vacation-manager/1.0/vacation-document?teamId=' + teamId,
            contentType : 'application/json'
        });
        var fetchTeamMembers = VM$.ajax({
            type : 'GET',
            url : AJS.contextPath() + '/rest/vacation-manager/1.0/team/' + teamId,
            contentType : 'application/json'
        });

        VM$.when(fetchVacations, fetchTeamMembers)
            .then(function (vacationsResponse, teamMembersResponse) {
                vacations = vacationsResponse[0];
                var users = teamMembersResponse[0].users;
                supervisors = teamMembersResponse[0].supervisors;
                users = users.concat(supervisors);
                teamMembers = users.filter(onlyUnique);

                displayTeamStatistics();
            });
    }

    function onlyUnique(value, index, self) {
        return self.indexOf(value) === index;
    }

    function displayTeamStatistics() {
        teamCalendar.fullCalendar('removeEventSources');
        if (issueKeyFromUrl) {
            var vacation = findVacationByIssueKey(issueKeyFromUrl);
            if (vacation) {
                teamCalendar.fullCalendar('gotoDate', VM$.moment(vacation.selectedPeriod.startDateTime, DATE_TIME_PARSE_FORMAT));
            }
        }

        var calendarEvents = [];
        vacations.forEach(function(vacation) {
            calendarEvents.push(buildVacationCalendarEvent(vacation));
        });

        getHolidaySchemesAssignedToTeamMembers().forEach(function (holidayScheme) {
            getOneTimeHolidaysFrom(holidayScheme).forEach(function (holiday) {
                calendarEvents.push(buildHolidayCalendarEvent(holiday));
            });
        });

        teamCalendar.fullCalendar('addEventSource', calendarEvents);
        displayYearlyHolidays();
    }

    function getHolidaySchemesAssignedToTeamMembers() {
        var holidaySchemesIdsAssignedToTeamMembers = [];
        teamMembers.forEach(function (member) {
            holidaySchemesIdsAssignedToTeamMembers.push(member.holidaySchemeId);
        });
        holidaySchemesIdsAssignedToTeamMembers = holidaySchemesIdsAssignedToTeamMembers.filter(onlyUnique);

        return holidaySchemes.filter(function (holidayScheme) {
            return holidaySchemesIdsAssignedToTeamMembers.indexOf(holidayScheme.id) > -1;
        })
    }

    function getOneTimeHolidaysFrom(holidayScheme) {
        return holidayScheme.holidays.filter(function (holiday) { return holiday.yearly === false; });
    }

    function findVacationByIssueKey(issueKey) {
        for (var i = 0; i < vacations.length; i++) {
            if (vacations[i].issueKey === issueKey) {
                return vacations[i];
            }
        }

        return null;
    }

    function getYearlyHolidaysFrom(holidayScheme) {
        return holidayScheme.holidays.filter(function (holiday) { return holiday.yearly; });
    }

    function displayYearlyHolidays() {
        getHolidaySchemesAssignedToTeamMembers().forEach(function (holidayScheme) {
            getYearlyHolidaysFrom(holidayScheme).forEach(function (holiday) {
                teamCalendar.fullCalendar('renderEvent', buildHolidayCalendarEvent(holiday), true);
            });
        });
    }

    function hideTeamSelection() {
        AJS.$('#statistics-team-team-selection').hide();
    }
    
    function displayTeamNameInHeader(team) {
        var currentHeaderText = AJS.$('header.aui-page-header h1').text();
        AJS.$('header.aui-page-header h1').text(currentHeaderText + ': ' + team.name);
    }

    function buildVacationCalendarEvent(vacation) {
        var startTime = VM$.moment(vacation.selectedPeriod.startDateTime, DATE_TIME_PARSE_FORMAT);
        var endTime = VM$.moment(vacation.selectedPeriod.endDateTime, DATE_TIME_PARSE_FORMAT);
        if (vacation.selectedPeriod.periodType === 'DAILY') {
            endTime.add(1, 'days');
        } else if (vacation.selectedPeriod.periodType === 'HALF_DAILY') {
            startTime.startOf('day');
            endTime.endOf('day');
            if (vacation.selectedPeriod.halfDayType === 'PM') {
                startTime.hours(12).minutes(0).seconds(0);
            } else {
                endTime.hours(12).minutes(0).seconds(0);
            }
        }

        return {
            backgroundColor: vacation.status === 'APPROVED' ? '#519fc5' : '#8fdbff',
            title: findTeamMemberById(vacation.reporter.id).displayName + ' (' + findVacationTypeById(vacation.vacationTypeId).name + ')',
            start: startTime,
            end: endTime,
            allDay: vacation.selectedPeriod.periodType === 'DAILY',
            vacationDocument: vacation
        };
    }
    
    function findTeamMemberById(userId) {
        for (var i = 0; i < teamMembers.length; i++) {
            if (teamMembers[i].id === userId) {
                return teamMembers[i];
            }
        }

        return null;
    }

    function findVacationTypeById(id) {
        for (var i = 0; i < vacationTypes.length; i++) {
            if (vacationTypes[i].id === id) {
                return vacationTypes[i];
            }
        }

        return null;
    }

    function buildHolidayCalendarEvent(holiday) {
        var startDate = VM$.moment(holiday.date);
        if (holiday.yearly) {
            startDate.year(teamCalendar.fullCalendar('getDate').year());
        }

        return {
            id: 'holiday-' + holiday.id,
            backgroundColor: '#c8f1b9',
            borderColor: '#257e4a',
            textColor: '#666666',
            title: holiday.description,
            start: startDate,
            allDay: true,
            yearly: holiday.yearly
        };
    }

    function onCalendarEventRender(event, element) {
        if (!event.vacationDocument) {
            return;
        }

        element.addClass('vacation');
        if (event.vacationDocument.issueKey === issueKeyFromUrl) {
            element.addClass('selected');
        }

        VM$(element).tooltipster({
            content: buildTooltip(event.vacationDocument),
            contentAsHTML: true,
            interactive: true,
            trigger: 'click'
        });

        var period = event.vacationDocument.selectedPeriod;
        if (period.periodType === 'HALF_DAILY') {
            element.addClass('vacation-half-day');
            element.toggleClass('vacation-half-day-pm', period.halfDayType === 'PM');
        } else if (period.periodType === 'HOURLY') {
            element.addClass('vacation-hourly');
        }
    }

    function buildTooltip(vacationDocument) {
        var reporter = findTeamMemberById(vacationDocument.reporter.id);

        var tooltipElement = VM$('<div>')
            .append(
                VM$('<div>').text(reporter.displayName)
            ).append(buildVacationTypeTooltipLabel(vacationDocument))
            .append(buildVacationDurationTooltipLabel(vacationDocument));

        var shouldShowIssueLink = loggedUserIsHR ||
            loggedInUser.id === vacationDocument.reporter.id ||
            isLoggedUserCurrentTeamSupervisor();
        if (shouldShowIssueLink) {
            tooltipElement.append(
                VM$('<div>').append(
                    VM$('<a>')
                        .attr('href', AJS.contextPath() + '/browse/' + vacationDocument.issueKey)
                        .attr('target', '_blank')
                        .text(AJS.I18n.getText('vacationmanager.action.statistics.team.open.vacation.document'))
                )
            )
        }

        return tooltipElement;
    }

    function buildVacationTypeTooltipLabel(vacationDocument) {
        var vacationTypeLabel = VM$('<div>').append(VM$('<span>').text(findVacationTypeById(vacationDocument.vacationTypeId).name));
        if (vacationDocument.status === 'OPEN') {
            vacationTypeLabel.append(VM$('<span>').text(' (' + AJS.I18n.getText('vacationmanager.action.statistics.team.document.status.not.approved') + ')'));
        }
        return vacationTypeLabel;
    }

    function buildVacationDurationTooltipLabel(vacationDocument) {
        var period = vacationDocument.selectedPeriod;
        var durationLabel = '';
        switch (period.periodType) {
            case 'DAILY':
                durationLabel = vmUtils.formatDays(vacationDocument.calculatedDays) + ' ('
                    + VM$.moment(period.startDateTime).format(DATE_DISPLAY_FORMAT) + ' - '
                    + VM$.moment(period.endDateTime).format(DATE_DISPLAY_FORMAT) + ')';
                break;
            case 'HALF_DAILY':
                var halfDayType = period.lafDayType === 'AM' ?
                    AJS.I18n.getText('vacationmanager.action.statistics.team.first.day.half') :
                    AJS.I18n.getText('vacationmanager.action.statistics.team.second.day.half');
                durationLabel = VM$.moment(period.startDateTime).format('DD-MM-YYYY') + ' (' + halfDayType  + ')';
                break;
            case 'HOURLY':
                durationLabel = vmUtils.formatTime(vacationDocument.calculatedMinutes) + ' ('
                    + VM$.moment(period.startDateTime).format('HH:mm') + ' - '
                    + VM$.moment(period.endDateTime).format('HH:mm') + ')';
                break;
        }

        return durationLabel;
    }

    function isLoggedUserCurrentTeamSupervisor() {
        var loggedUserName = loggedInUser.name;
        for (var i = 0; i < supervisors.length; i++) {
            if (supervisors[i].name === loggedUserName) {
                return true;
            }
        }

        return false;
    }

});
