function vacationKindStatisticsFilters(params) {
    var statisticsTable = params.statisticsTable;
    var columnIndexes = params.columnIndexes;
    return {
        userFilter: userFilter(),
        availableDaysFilter: availableDaysFilter(),
        availableMinutesFilter: availableMinutesFilter(),
        carryoverDaysFilter: carryoverDaysFilter(),
        carryoverMinutesFilter: carryoverMinutesFilter(),
        usedDaysFilter: usedDaysFilter(),
        usedMinutesFilter: usedMinutesFilter(),
        teamFilter: teamFilter()
    };

    function userFilter() {
        return {
            name: AJS.I18n.getText('vacationmanager.action.statistics.filter.user'),
            filterFunction: function (row, expected) {
                return row[columnIndexes.user].toLowerCase().indexOf(expected.value.toLowerCase()) > -1;
            },
            template: VacationManager.Templates.Filter.filterText({name: AJS.I18n.getText('vacationmanager.action.statistics.filter.user')}),
            getExpectedFromTemplate: function (template) {
                return {
                    value: template.find('input[name=value]').val()
                };
            },
            getValueForLabel: function (expected) {
                if (!expected.value) {
                    return '';
                }

                return ': ' + expected.value;
            }
        };
    }

    function availableDaysFilter() {
        return abstractNumericFilter({
            name: AJS.I18n.getText('vacationmanager.action.statistics.filter.available.days'),
            columnIndex: columnIndexes.availableDaysOrderData
        });
    }

    function availableMinutesFilter() {
        return abstractNumericFilter({
            name: AJS.I18n.getText('vacationmanager.action.statistics.filter.available.minutes'),
            columnIndex: columnIndexes.availableMinutesOrderData
        });
    }

    function carryoverDaysFilter() {
        return abstractNumericFilter({
            name: AJS.I18n.getText('vacationmanager.action.statistics.filter.carryover.days'),
            columnIndex: columnIndexes.carryOverDaysOrderData
        });
    }

    function carryoverMinutesFilter() {
        return abstractNumericFilter({
            name: AJS.I18n.getText('vacationmanager.action.statistics.filter.carryover.minutes'),
            columnIndex: columnIndexes.carryOverMinutesOrderData
        });
    }

    function usedDaysFilter() {
        return abstractNumericFilter({
            name: AJS.I18n.getText('vacationmanager.action.statistics.filter.used.days'),
            columnIndex: columnIndexes.usedDaysOrderData
        });
    }

    function usedMinutesFilter() {
        return abstractNumericFilter({
            name: AJS.I18n.getText('vacationmanager.action.statistics.filter.used.minutes'),
            columnIndex: columnIndexes.usedMinutesOrderData
        });
    }

    function abstractNumericFilter(params) {
        return {
            name: params.name,
            filterFunction: numericFilterFunctionForColumn(params.columnIndex),
            template: VacationManager.Templates.Filter.filterNumeric({name: params.name}),
            getExpectedFromTemplate: function (template) {
                var value = template.find('input[name=value]').val();

                return {
                    value: value ? parseInt(value) : null,
                    operator: template.find('select[name=operator] option:selected').val(),
                    showEmpty: template.find('[name=empty-values]').is(':checked')
                };
            },
            getValueForLabel: getValueForLabelForNumericFilter
        };
    }

    function teamFilter() {
        return {
            name: AJS.I18n.getText('vacationmanager.action.statistics.filter.team'),
            filterFunction: function (row, expected) {
                var expectedTeamNames = expected.value;
                for (var j = 0; j < expectedTeamNames.length; j++) {
                    expectedTeamNames[j] = expectedTeamNames[j].toLowerCase();
                }
                var teamNames = row[columnIndexes.teamNames].split(',');
                for (var i = 0; i < teamNames.length; i++) {
                    if (expectedTeamNames.indexOf(teamNames[i].toLowerCase()) > -1) {
                        return true;
                    }
                }

                return false;
            },
            template: VacationManager.Templates.Filter.filterTeam({name: AJS.I18n.getText('vacationmanager.action.statistics.filter.team')}),
            getExpectedFromTemplate: function (template) {
                return {
                    value: template.find('#team-names-select').val()
                };
            },
            getValueForLabel: function (expected) {
                if (!expected.value || expected.value.length === 0) {
                    return '';
                }

                return ': ' + expected.value.join(', ');
            },
            initializeTemplate: function (template) {
                getTeamNamesFromStatisticsTable()
                    .forEach(function(teamName) {
                        template.find('#team-names-select').append(AJS.$('<option>').val(teamName).text(teamName))
                    });
                template.find('#team-names-select').auiSelect2({
                    placeholder: AJS.I18n.getText('vacationmanager.action.statistics.filter.team.placeholder')
                });
            }
        };
    }

    function numericFilterFunctionForColumn(columnIndex) {
        return function (row, expected) {
            if (row[columnIndex] === null || isNaN(row[columnIndex]) || row[columnIndex] === '') {
                return expected.showEmpty;
            }

            switch (expected.operator) {
                case 'equal':
                    return row[columnIndex] === expected.value;
                case 'less':
                    return row[columnIndex] < expected.value;
                case 'more':
                    return row[columnIndex] > expected.value;
                default:
                    throw new Error('unsupported operator: ' + expected.operator);
            }
        };
    }

    function getValueForLabelForNumericFilter(expected) {
        if (expected.value === null) {
            return '';
        }

        switch (expected.operator) {
            case 'equal':
                return ' = ' + expected.value;
            case 'less':
                return ' < ' + expected.value;
            case 'more':
                return ' > ' + expected.value;
            default:
                throw new Error('unsupported operator: ' + expected.operator);
        }
    }

    function getTeamNamesFromStatisticsTable() {
        var allTeamsInTable = statisticsTable.columns(columnIndexes.teamNames).data().eq(0);
        var availableTeams = [];
        for (var i = 0; i < allTeamsInTable.length; i++) {
            availableTeams = availableTeams.concat(allTeamsInTable[i].split(','));
        }

        return availableTeams.filter(onlyUnique)
    }

    function onlyUnique(value, index, self) {
        return self.indexOf(value) === index;
    }
}