function VacationKindStatisticsFiltersController(params) {
    var statisticsTable = params.statisticsTable;
    var availableFilters = vacationKindStatisticsFilters(params);
    var activeFilters = [];

    VM$.fn.dataTable.ext.search.push(checkAllActiveFilters);

    AJS.$('.add-filter-button').click(onAddFilterButtonClick);

    function onAddFilterButtonClick() {
        var filterName = AJS.$(this).attr('data-filter-to-activate');
        activateFilter(filterName);
        hideButtonForFilter(filterName);
    }

    function activateFilter(filterName) {
        var activeFilter = new ActiveFilter({
            filter: availableFilters[filterName]
        });
        activeFilters.push(activeFilter);

        buildDomElementsForFilter(activeFilter);
        refilterStatisticsTable();
    }

    function checkAllActiveFilters(settings, row) {
        return activeFilters.every(function (activeFilter) {
            return activeFilter.apply(row);
        });
    }

    function buildDomElementsForFilter(activeFilter) {
        var filterLabel = createAndShowFilterLabel(activeFilter);
        activeFilter.labelElement = filterLabel;

        createAndShowFilterPopup(activeFilter, filterLabel);
    }

    function createAndShowFilterLabel(activeFilter) {
        var filterLabel = AJS.$(VacationManager.Templates.Filter.filterLabel({name: activeFilter.filter.name}))
            .appendTo('#vacman-filters');
        filterLabel.find('.remove-filter').click(function() {
            deactivateFilter(activeFilter);
            showButtonForFilter(activeFilter.filter);
        });

        return filterLabel;
    }

    function createAndShowFilterPopup(activeFilter, filterLabelElement) {
        var popupTrigger = filterLabelElement;
        var popupId = 'filter-popup';
        var popupConfig = {
            'hideCallback': onPopupHide,
            'initCallback': onPopupInitialized,
            'hideDelay': null,
            'width': 350
        };

        function onPopupShow(content, trigger, showPopup) {
            if (content.children().length) {
                return;
            }

            content.html(activeFilter.filter.template)
                .click(onFilterPopupClick);
            if (activeFilter.filter.initializeTemplate) {
                activeFilter.filter.initializeTemplate(content);
            }
            showPopup();
        }

        function onPopupHide() {
            activeFilter.readExpectedValue(inlineDialog);
            activeFilter.updateLabel();
            refilterStatisticsTable();
        }

        function onPopupInitialized() {
            setTimeout(function() {inlineDialog.find('input[name=value]').focus();}, 150);
        }

        var inlineDialog = AJS.InlineDialog(popupTrigger, popupId, onPopupShow, popupConfig);
        inlineDialog.show();
    }

    function onFilterPopupClick(event) {
        if (AJS.$(event.target).attr('name') !== 'filter-ok') {
            event.stopPropagation();
        }
    }

    function refilterStatisticsTable() {
        statisticsTable.rows().invalidate();
        statisticsTable.draw();
    }

    function hideButtonForFilter(filterName) {
        AJS.$('[data-filter-to-activate=' + filterName + ']').hide();
    }

    function showButtonForFilter(filter) {
        for (var filterName in availableFilters) {
            if (availableFilters[filterName] === filter) {
                AJS.$('[data-filter-to-activate=' + filterName + ']').show();
                break;
            }
        }
    }

    function deactivateFilter(activeFilter) {
        activeFilters = activeFilters.filter(function (filter) {
            return filter !== activeFilter;
        });

        activeFilter.labelElement.remove();
        refilterStatisticsTable();
    }

    function ActiveFilter(params) {
        this.filter = params.filter;
        this.expected = { value: null };
    }
    ActiveFilter.prototype.apply = function (row) {
        return this.expected.value === null || this.filter.filterFunction(row, this.expected);
    };
    ActiveFilter.prototype.readExpectedValue = function (template) {
        this.expected = this.filter.getExpectedFromTemplate(template);
    };
    ActiveFilter.prototype.updateLabel = function () {
        this.labelElement.find('[name=expected-value]')
            .text(this.filter.getValueForLabel(this.expected));
    };
}