var statisticsData = {
	instances: []
};

AJS.$(document).ready(function() {

    var TableColumnsEnum = {
        userId: 0,
        user: 1,
        startDate: 2,
        endDate: 3,
        availableTime: 4,
        carryOverTime: 5,
        usedTime: 6,
        availableDaysOrderData: 7,
        availableMinutesOrderData: 8,
        carryOverDaysOrderData: 9,
        carryOverMinutesOrderData: 10,
        usedDaysOrderData: 11,
        usedMinutesOrderData: 12,
        teams: 13,
        teamNames: 14,
        documents: 15,
        documentsOrderData: 16
    };
	
	var checkDateStart=new Date();
	var checkDateEnd=new Date(new Date().getFullYear(), 11, 31);
	var vacationKindId=0;
	
	AJS.$('.aui-date-picker').each(function() {
		AJS.$(this).datePicker({'overrideBrowserDefault': true});
	});

	AJS.$("#vacationKind").change(function() {
		vacationKindChange(AJS.$(this).val());
	});

	function vacationKindChange(id) {
		vacationKindId=id;
		AJS.$("[name=periodSelectContainer]").hide();
		AJS.$("[name=periodSelectContainer][vacationKindId="+id+"]").show();
		var obj = AJS.$("[name=periodSelect][vacationKindId="+id+"]");
		selectObjectDate(obj, checkDateStart, checkDateEnd);
		var oldCheckStart=checkDateStart;
		var oldCheckEnd=checkDateEnd;
		obj.change();
		checkDateStart=oldCheckStart;
		checkDateEnd=oldCheckEnd;
	}
	
	function selectObjectDate(obj, dateStart, dateEnd) {
		if (obj.prop("tagName").toUpperCase()=="SELECT") {
			var childrens=obj.children();
			var children=null;
			for (var i=0; i<childrens.length; i++) {
				children=AJS.$(childrens[i]);
				var start=new Date(children.val().split(",")[0]);
				var end=new Date(children.val().split(",")[1]);
				if (obj.length>1) {
					start=AJS.$("[name=periodSelectContainer][vacationKindId="+vacationKindId+"]").find(".dateStart[name=periodSelect]").val();
					end=AJS.$("[name=periodSelectContainer][vacationKindId="+vacationKindId+"]").find(".dateEnd[name=periodSelect]").val();
				}
				if (dateStart<=end) {
					obj.val(children.val());
					return;
				}
			}
			obj.val(children.val());
			return;
		} else {
			if (obj.length>1) {
				AJS.$(obj[0]).val(parseDate(dateStart));
				AJS.$(obj[1]).val(parseDate(dateEnd));
			} else {
				AJS.$(obj[0]).val(parseDate(dateStart));
			}
		}
	}

	function parseDate(date) {
		var year=date.getFullYear();
		var month=pad((date.getMonth()+1),2);
		var day=pad((date.getDate()),2);
		return year+"-"+month+"-"+day;
	}
	
	function pad(num, size) {
	    var s = "00000000000000" + num;
	    return s.substr(s.length-size);
	}

	AJS.$('.overlay-spinner').spin();

	AJS.$('[name=periodSelect]').each(function() {
		var obj = AJS.$(this);
		obj.change(function() {
			var dates=obj.val().split(",");
			var start=dates[0];
			var end=dates[0];
			if (dates.length>1) {
				end=dates[1];
			} else {
				start=AJS.$("[name=periodSelectContainer][vacationKindId="+vacationKindId+"]").find(".dateStart[name=periodSelect]").val();
				end=AJS.$("[name=periodSelectContainer][vacationKindId="+vacationKindId+"]").find(".dateEnd[name=periodSelect]").val();
			}
			checkDateStart=new Date(start);
			checkDateEnd=new Date(end);
			changeTableData(AJS.$("#vacationKind").val(), start, end);
		});
	});
	
	function changeTableData(vacationKindId, start, end) {

		statisticsData.vacationKindName = AJS.$("#vacationKind option:selected").text();
		statisticsData.start = start;
		statisticsData.end = end;

		hideErrors();
		AJS.$("#loading-overlay").show();
		jQuery.ajax({
			type : "GET",
			url : AJS.contextPath() + "/rest/vacation-manager/1.0/statistics/vacation-type/" + vacationKindId +
			"?startDate=" + VM$.formatDateIso(start) + "&endDate=" + VM$.formatDateIso(end),
			async : true
		}).done(function(data) {
			statisticsData.instances = data.userInstances;
			populateTableData(data);
		}).fail(function(data) {
			showErrors(VM$.parseJSON(data.responseText).errors);
		}).always(function() {
			AJS.$("#loading-overlay").hide();
		});
	}
	
	function populateTableData(json) {
		var userInstances = json.userInstances;
											
		table.rows().every(function(id) {
			var row=this;
			var data=row.data();
			var userId=data[TableColumnsEnum.userId];
			clearRow(row);
			var userInstance=userInstances[userId];
			if (userInstance !== undefined) {
				var instances=userInstance.instances;
				for (var inst=0; inst<instances.length; inst++) {
					var instance=instances[inst];
					addRowData(row, {
                        vacationKind: json.vacationType,
						startDate: new Date(instance.startDate),
						endDate: new Date(instance.endDate),
						availableDays: instance.availableDays,
                        availableMinutes: instance.availableMinutes,
                        carryOverDays: instance.carryOverDays,
                        carryOverMinutes: instance.carryOverMinutes,
						usedDays: instance.usedDays,
                        usedMinutes: instance.usedMinutes,
						teams: userInstance.teams,
                        vacationDocuments: instance.vacationDocuments
					});
				}
				if (instances.length === 0) {
					addRowData(row, {vacationKind: json.vacationType, teams: userInstance.teams});
				}
			}
		});
		
		table.draw();
	}
	
	function clearRow(row) {
		var data=row.data();
        for (var i = TableColumnsEnum.user + 1; i <= TableColumnsEnum.documentsOrderData; i++) {
            data[i]="";
        }
		row.data(data);
	}
	
	function addRowData(row, userVacationInstance) {
		var data=row.data();
		var vacationKind = userVacationInstance.vacationKind;
		
        var notApplicableLabel = "<div value='" + AJS.I18n.getText('vacationmanager.text.not.applicable') + "' class='not-applicable-element'>" + AJS.I18n.getText('vacationmanager.text.not.applicable') + "</div>";
        var noDayLimitLabel = "<div value='" + AJS.I18n.getText('vacationmanager.vacationkind.no.day.limit') + "' class='not-applicable-element'>" + AJS.I18n.getText("vacationmanager.vacationkind.no.day.limit") + "</div>";
        var noData = "<div value='-' class='not-applicable-element'>-</div>";

        var startDateLabel = noData;
		if (userVacationInstance.startDate) {
            startDateLabel = getDateLabel(userVacationInstance.startDate);
		}

		var endDateLabel = noData;
        if (vacationKind.hasEndDate === false && vacationKind.kindType === 'ONE_TIME') {
            endDateLabel = notApplicableLabel;
        } else if (userVacationInstance.endDate) {
            endDateLabel = getDateLabel(userVacationInstance.endDate);
		}

        var availableTimeLabel = noData;
        if (vacationKind.hasDayLimit === false) {
            availableTimeLabel = noDayLimitLabel;
        } else if (data[TableColumnsEnum.availableDaysOrderData] !== undefined || data[TableColumnsEnum.availableMinutesOrderData] !== undefined) {
            data[TableColumnsEnum.availableDaysOrderData] = (data[TableColumnsEnum.availableDaysOrderData] ? parseFloat(data[TableColumnsEnum.availableDaysOrderData]) : 0) + userVacationInstance.availableDays;
            data[TableColumnsEnum.availableMinutesOrderData] = parseInt(data[TableColumnsEnum.availableMinutesOrderData], 10) + userVacationInstance.availableMinutes;
            availableTimeLabel = "<div>" + vmUtils.formatDaysAndTime(userVacationInstance.availableDays, userVacationInstance.availableMinutes) + "</div>";
		}

		var carryOverTimeLabel;
        if (vacationKind.hasCarryOver === false) {
            carryOverTimeLabel = notApplicableLabel
		} else if (userVacationInstance.carryOverDays === undefined && userVacationInstance.carryOverMinutes === undefined) {
            carryOverTimeLabel = noData;
		} else {
            data[TableColumnsEnum.carryOverDaysOrderData] = parseFloat(data[TableColumnsEnum.carryOverDaysOrderData]) + userVacationInstance.carryOverDays;
            data[TableColumnsEnum.carryOverMinutesOrderData] = parseInt(data[TableColumnsEnum.carryOverMinutesOrderData], 10) + userVacationInstance.carryOverMinutes;
            carryOverTimeLabel = "<div value='" + userVacationInstance.carryOverDays + "'>" + vmUtils.formatDaysAndTime(userVacationInstance.carryOverDays, userVacationInstance.carryOverMinutes) + "</div>";
        }

        var usedTimeLabel = noData;
		if (userVacationInstance.usedDays !== undefined || userVacationInstance.usedMinutes !== undefined) {
			var usedDays = userVacationInstance.usedDays || 0;
			var usedMinutes = userVacationInstance.usedMinutes || 0;
			data[TableColumnsEnum.usedDaysOrderData] = (data[TableColumnsEnum.usedDaysOrderData] || 0) + usedDays;
			data[TableColumnsEnum.usedMinutesOrderData] = (data[TableColumnsEnum.usedMinutesOrderData] || 0) + usedMinutes;
			
            usedTimeLabel = "<div>" + vmUtils.formatDaysAndTime(usedDays, usedMinutes) + "</div>";
		}

		data[TableColumnsEnum.startDate] += startDateLabel;
		data[TableColumnsEnum.endDate] += endDateLabel;
		data[TableColumnsEnum.availableTime] += availableTimeLabel;
		data[TableColumnsEnum.carryOverTime] += carryOverTimeLabel;
		data[TableColumnsEnum.usedTime] += usedTimeLabel;

		var documentsCount = 0;
		var documentsLink = VacationManager.Templates.UserStatistics.documentsJqlLink({
			vacationDocuments: userVacationInstance.vacationDocuments,
		    jqlSearchUrl: AJS.params.baseURL + '/issues/?jql='
		});
		if (documentsLink.trim() === '') {
			documentsLink = noData;
		} else {
			documentsCount = userVacationInstance.vacationDocuments.length;
		}

		if (userVacationInstance.teams) {
            var teamNames = [];
            for (var i = 0; i < userVacationInstance.teams.length; i++) {
            	teamNames.push(userVacationInstance.teams[i].name);
			}

            data[TableColumnsEnum.teams] = teamNames.join(', ');
            data[TableColumnsEnum.teamNames] = teamNames;
		}

		data[TableColumnsEnum.documents] = "<div value='"+documentsCount+"'>" + documentsLink + "</div>";
		data[TableColumnsEnum.documentsOrderData] = '';
		if (userVacationInstance.vacationDocuments) {
			data[TableColumnsEnum.documentsOrderData] = userVacationInstance.vacationDocuments.length;
		}
		
		row.data(data);
		row.invalidate();
	}

	function getDateLabel(date) {
		return '<div value="' + VM$.moment(date).format('YYYY-MM-DD') + '">' + VM$.moment(date).format(JIRA.translateSimpleDateFormat(AJS.Meta.get('date-dmy'))) + '</div>';
    }

	var table = VM$('#vacationDefinitionsTable').DataTable({
		language: datatablesLangObject,
		order: [ [TableColumnsEnum.user, 'asc'] ],
		columnDefs: [
		   { targets: [TableColumnsEnum.startDate, TableColumnsEnum.endDate, TableColumnsEnum.teams, TableColumnsEnum.documents, TableColumnsEnum.documentsOrderData], orderable: false },
		   { visible: false,
		     targets: [
			   TableColumnsEnum.userId,
			   TableColumnsEnum.availableDaysOrderData,
			   TableColumnsEnum.availableMinutesOrderData,
			   TableColumnsEnum.carryOverDaysOrderData,
			   TableColumnsEnum.carryOverMinutesOrderData,
			   TableColumnsEnum.usedDaysOrderData,
			   TableColumnsEnum.usedMinutesOrderData,
			   TableColumnsEnum.teamNames,
			   TableColumnsEnum.documentsOrderData]},
		   { targets: [TableColumnsEnum.availableTime], orderData: [TableColumnsEnum.availableDaysOrderData, TableColumnsEnum.availableMinutesOrderData] },
		   { targets: [TableColumnsEnum.carryOverTime], orderData: [TableColumnsEnum.carryOverDaysOrderData, TableColumnsEnum.carryOverMinutesOrderData] },
		   { targets: [TableColumnsEnum.usedTime], orderData: [TableColumnsEnum.usedDaysOrderData, TableColumnsEnum.usedMinutesOrderData] }
		]
	});
	AJS.$("#vacationDefinitionsTable_filter").hide();

	statisticsData.table = table;
	statisticsData.vacationKindName = null;
	statisticsData.start = null;
	statisticsData.end = null;

    new VacationKindStatisticsFiltersController({
        statisticsTable: table,
        columnIndexes: TableColumnsEnum
    });

	vacationKindChange(AJS.$("#vacationKind").val());
	
	function showErrors(reponseErrors) {
        AJS.$("#errors").html(VacationManager.Templates.Messages.restResponseErrors({
            "errorCollection": reponseErrors
        }));
	}
	
	function hideErrors() {
		AJS.$("#errors").empty();
	}

	window.onbeforeunload=null;
});
