AJS.$(document).ready(function() {
    AJS.$('#exportFiltered').click(onExportButtonClick);

    function onExportButtonClick() {
        try {
            exportToCsv();
        } catch (err) {
            alert(AJS.I18n.getText('vacationmanager.action.statistics.export.csv.error', err));
        }
    }

    function exportToCsv() {
        var rawData = encodeCsvLine(getColumnNames());

        var allVacationInstances = AJS.$.map(getTableRows(), getUserVacationInstancesFromTableRow);
        rawData += allVacationInstances
            .map(getCsvLineFromInstance)
            .map(encodeCsvLine)
            .join('');

        var blob = new Blob([rawData], {type: 'text/plain;charset=utf-8'});
        saveAs(blob, generateFileName(statisticsData));
    }

    function getColumnNames() {
        return [
            AJS.I18n.getText('vacationmanager.action.statistics.export.csv.column.vacation.kind'),
            AJS.I18n.getText('vacationmanager.text.full.name'),
            AJS.I18n.getText('vacationmanager.text.username'),
            AJS.I18n.getText('vacationmanager.text.start.date'),
            AJS.I18n.getText('vacationmanager.text.end.date'),
            AJS.I18n.getText('vacationmanager.text.available.days'),
            AJS.I18n.getText('vacationmanager.text.available.minutes'),
            AJS.I18n.getText('vacationmanager.text.carryover.days'),
            AJS.I18n.getText('vacationmanager.text.carryover.minutes'),
            AJS.I18n.getText('vacationmanager.text.used.days'),
            AJS.I18n.getText('vacationmanager.text.used.minutes'),
            AJS.I18n.getText('vacationmanager.action.statistics.user.used.days.documents')
        ];
    }

    function encodeCsvLine(elements) {
        var line = '';
        line += elements
            .map(wrapWithQuotes)
            .join(',');
        line += '\r\n';
        return line;
    }

    function wrapWithQuotes(text) {
        return '"' + text + '"';
    }

    function getTableRows() {
        return statisticsData.table.$('tr', {filter: 'applied'});
    }

    function getUserVacationInstancesFromTableRow(row) {
        var userId = AJS.$(row).attr('userid');
        var instances = statisticsData.instances[userId].instances;
        instances.forEach(function (instance) {
            instance.fullName = AJS.$(row).attr('fullname');
            instance.userName = AJS.$(row).attr('username');
        });

        if (instances.length === 0) {
            return [instanceWithNoData(row)];
        }

        return instances;
    }

    function instanceWithNoData(row) {
        return {
            vacationKindName: statisticsData.vacationKindName,
            fullName: AJS.$(row).attr('fullname'),
            userName: AJS.$(row).attr('username'),
            availableDays: '-',
            availableMinutes: '-',
            carryOverDays: '-',
            carryOverMinutes: '-',
            usedDays: '-',
            usedMinutes: '-',
            vacationDocuments: []
        };
    }

    function getCsvLineFromInstance(userVacationInstance) {
        var startDate = '-';
        var endDate = '-';
        if (userVacationInstance.startDate) {
            startDate = VM$.moment(userVacationInstance.startDate).format('YYYY-MM-DD');
        }
        if (userVacationInstance.endDate) {
            endDate = VM$.moment(userVacationInstance.endDate).format('YYYY-MM-DD')
        }

        return [
            statisticsData.vacationKindName,
            userVacationInstance.fullName,
            userVacationInstance.userName,
            startDate,
            endDate,
            userVacationInstance.availableDays,
            userVacationInstance.availableMinutes,
            userVacationInstance.carryOverDays,
            userVacationInstance.carryOverMinutes,
            userVacationInstance.usedDays,
            userVacationInstance.usedMinutes,
            userVacationInstance.vacationDocuments.length
        ];
    }

    function generateFileName(statisticsData) {
        return 'statistics_' + statisticsData.vacationKindName + '_' + statisticsData.start + '_' + statisticsData.end + '.csv';
    }

});
