function VacationManagerChart(_container, issueContainerUtils) {
    var container = _container.find("[name=vm_chart]");
    var chartId = container.attr("chart-id");
    var formBody = _container;
    var fieldGroup = container.closest(".field-group");

    var vacationTimeChart;
    var chartCfId;
    var vacationTypeId;
    var issueId;
    var reporter;

    var validation = "";
    var errors = null;

    var availableDays = 0;
    var remainingDays = 0;
    var entitledDays = 0;
    var leftoverDays = 0;
    var usedDays = 0;
    var usedMinutes = 0;
    var reservedDays = 0;
    var usedByThisIssueDays = 0;
    var selectedDays = 0;
    var selectedMinutes = 0;
    var availableMinutes = 0;
    var remainingMinutes = 0;
    var requestedHalfDay = "AM";

    var hasDayLimit;

    var periodStart;
    var periodEnd;

    var vmMessages;
    var chartLoaded = false;

    function init() {
        initMsgContainers();
        initValuesFromDocument();

        vacationTimeChart = new VacationDaysPieChart({
            chartSelector: '[name=chart_div]'
        });
        vacationTimeChart.onLoad(function () {
            chartLoaded = true;
            updateChart();
        });

        issueContainerUtils.getAllPeriodTypeOptions().on('change', onFieldsChanged);
        issueContainerUtils.getStartDateField().on('change', function() {
            var startDate = issueContainerUtils.getStartDateField().val();
            var endDate = issueContainerUtils.getEndDateField().val();
            if (!endDate) {
                issueContainerUtils.getEndDateField().val(startDate);
            }
            onFieldsChanged();
        });
        issueContainerUtils.getEndDateField().on('change', function() {
            var startDate = issueContainerUtils.getStartDateField().val();
            var endDate = issueContainerUtils.getEndDateField().val();
            if (!startDate) {
                issueContainerUtils.getStartDateField().val(endDate);
            }
            onFieldsChanged();
        });
        issueContainerUtils.getReporterField().on('change', onFieldsChanged);
        issueContainerUtils.getStartTimeField().on('change', onFieldsChanged);
        issueContainerUtils.getEndTimeMinutesField().on('change', onFieldsChanged);
        issueContainerUtils.getHalfOfDayField().change(function() {
            // TODO get rid of try-catch and prevent errors caused by missing custom fields VM-503
        	try {
        		requestedHalfDay = getSelectedHalfOfDay(this.value);
        	} catch(error) {
        		console.error(error);
        	}
        	onFieldsChanged();
        });

        onFieldsChanged();
    }

    function initMsgContainers() {

        var newFieldGroup = $("<div class='field-group'></div>");
        vmMessages = $("<div class='error'></div>");

        newFieldGroup.append(vmMessages);
        fieldGroup.after(newFieldGroup);

        var hintText = AJS.I18n.getText("vacationmanager.vacation.customfield.statistics.send.to.supervisor");
        formBody.next().append(
                '<div id="vm-info" class="aui-dialog2-footer-hint" style="text-align:left;">' + hintText + '</div>');
    }

    function updateChart() {
    	if (!chartLoaded) {
    		return;
    	}

        if (errors != null) {
            fieldGroup.hide();
            var message = "";
            for (var i = 0; i < errors.length; i++) {
                message += "<div>" + errors[i].message + "</div>";
            }
            showWarningMessage(message);
            return;
        }

        remainingDays = availableDays - selectedDays - reservedDays;
        if (usedDays < 0) {
            usedDays = 0;
        }

        if (remainingDays < 0)
            remainingDays = 0;

        if (hasDayLimit == false)
            entitledDays = 0;

        if (hasDayLimit === false) {
            fieldGroup.hide();
        } else if (remainingDays <= 0 && reservedDays <= 0 && usedDays <= 0 && selectedDays <= 0) {
            if (hasDayLimit == true) {
                validation = AJS.I18n.getText("vacationmanager.vacation.validator.no.available.days.in.period");
            }
            fieldGroup.hide();
        } else {
            if (periodStart != "" && periodEnd != "") {
                var periodTypeInfo = "";
                var now = new Date();
                now.setHours(0);
                now.setMinutes(0);
                now.setSeconds(0);
                now.setMilliseconds(0);
                if (periodEnd < periodStart) {
                    periodTypeInfo = " <span style='color:#d04437;'>("
                            + AJS.I18n.getText("vacationmanager.vacation.customfield.statistics.period.invalid")
                            + ")</span>";
                } else if (periodStart > now) {
                    periodTypeInfo = " <span style='color:#14892c;'>("
                            + AJS.I18n.getText("vacationmanager.vacation.customfield.statistics.period.future")
                            + ")</span>";
                } else if (periodEnd < now) {
                    periodTypeInfo = " <span style='color:#f6c342;'>("
                            + AJS.I18n.getText("vacationmanager.vacation.customfield.statistics.period.past")
                            + ")</span>";
                }
                container.find(".periodInfo").html(
                        AJS.I18n.getText("vacationmanager.vacation.customfield.statistics.period", VM$
                                .formatDate(periodStart), VM$.formatDate(periodEnd))
                                + periodTypeInfo);
                container.find(".periodInfo").show();
            } else {
                container.find(".periodInfo").hide();
            }

            if (reporter != null) {
                container.find(".reporterInfo").text(
                        AJS.I18n.getText("vacationmanager.vacation.customfield.statistics.reporter", reporter));
                container.find(".reporterInfo").show();
            } else {
                container.find(".reporterInfo").hide();
            }

            fieldGroup.show();

            vacationTimeChart.draw({
                remainingDays: remainingDays,
                leftoverDays: leftoverDays,
                selectedDays: selectedDays,
                usedDays: usedDays,
                reservedDays: reservedDays
            });

            updateChartLabels({
                availableDays: availableDays - reservedDays,
                availableMinutes: availableMinutes,
                selectedDays: usedByThisIssueDays,
                selectedMinutes: selectedMinutes,
                remainingDays: remainingDays,
                remainingMinutes: remainingMinutes
            });
            showChartLabelsReload();
        }

        showWarningMessage(validation);
    }

    function updateChartLabels(params) {
        var availableTimeLabel = '';
        var remainingTimeLabel = '';
        
        if (selectedDays === 0 && selectedMinutes === 0) {
            availableTimeLabel = AJS.$('<div>')
                .append(AJS.$('<span>').text(AJS.I18n.getText('vacationmanager.vacation.customfield.statistics.available.time') + ': '))
                .append(AJS.$('<strong>').css('whiteSpace', 'nowrap').text(vmUtils.formatDaysAndTime(params.availableDays, params.availableMinutes)));
        } else {
            availableTimeLabel = AJS.$('<div>')
                .append(AJS.$('<span>').text(AJS.I18n.getText('vacationmanager.vacation.customfield.statistics.use.time') + ' '))
                .append(AJS.$('<strong>').css('whiteSpace', 'nowrap').text(vmUtils.formatDaysAndTime(params.selectedDays, params.selectedMinutes)))
                .append(AJS.$('<span>').text(' ' + AJS.I18n.getText('vacationmanager.vacation.customfield.statistics.from.available') + ' '))
                .append(AJS.$('<strong>').css('whiteSpace', 'nowrap').text(vmUtils.formatDaysAndTime(params.availableDays, params.availableMinutes)));

            remainingTimeLabel = AJS.$('<div>')
                .append(AJS.$('<span>').text(AJS.I18n.getText('vacationmanager.vacation.customfield.statistics.remaining.time') + ' '))
                .append(AJS.$('<strong>').css('whiteSpace', 'nowrap').text(vmUtils.formatDaysAndTime(params.remainingDays, params.remainingMinutes)));
        }

        container.find('[name=chart_availableTimeLabel]').html(availableTimeLabel);
        container.find('[name=chart_remainingTimeLabel]').html(remainingTimeLabel);
    }

    function showChartLabelsReload() {
        container.find('[name=reload-spinner]').show();
        container.find('[name=chart_time_labels]').hide();

        setTimeout(function () {
            container.find('[name=reload-spinner]').hide();
            container.find('[name=chart_time_labels]').show();
        }, 200);
    }
    
    function initValuesFromDocument() {
        chartCfId = container.find("[name=chart_chartCfId]").val();

        selectedDays = parseFloat(container.find("[name=chart_selectedDays]").val());
        if (!selectedDays) {
            selectedDays = 0;
        }
        availableDays = parseFloat(container.find("[name=chart_availableDays]").val());
        entitledDays = parseFloat(container.find("[name=chart_entitledDays]").val());
        leftoverDays = parseFloat(container.find("[name=chart_leftoverDays]").val());
        usedDays = parseFloat(container.find("[name=chart_usedDays]").val());
        reservedDays = parseFloat(container.find("[name=chart_reservedDays]").val());
        usedByThisIssueDays = parseFloat(container.find("[name=chart_daysUsedByThisIssue]").val());
        availableMinutes = parseInt(container.find("[name=chart_availableMinutes]").val(), 10);
        remainingMinutes = availableMinutes;
        usedMinutes = parseInt(container.find("[name=chart_usedMinutes]").val(), 10);

        hasDayLimit = (container.find("[name=chart_hasDayLimit]").val() == "true");

        vacationTypeId = container.find("[name=chart_vacationKindId]").val();

        periodStart = new Date(container.find("[name=chart_periodStart]").val());
        periodEnd = new Date(container.find("[name=chart_periodEnd]").val());

        issueId = container.find("[name=chart_issue_id]").val();

        validation = container.find("[name=chart_validation]").val();

        reporter = null;
    }

    function parseValuesFromJson(json) {
        selectedDays = parseFloat(json.selectedDays) || 0;
        selectedMinutes = parseInt(json.selectedMinutes, 10) || 0;
        entitledDays = parseFloat(json.entitledDays);
        availableDays = parseFloat(json.availableDays);
        leftoverDays = parseFloat(json.leftoverDays);
        usedDays = parseFloat(json.usedDays);
        reservedDays = parseFloat(json.reservedDays);
        usedByThisIssueDays = parseFloat(json.usedByThisIssueDays);
        availableMinutes = parseInt(json.availableMinutes, 10);
        remainingMinutes = parseInt(json.remainingMinutes, 10);
        usedMinutes = parseInt(json.usedMinutes, 10);

        periodStart = new Date(json.periodStart);
        periodEnd = new Date(json.periodEnd);

        validation = json.validation;

        if (json.reporter != undefined && json.reporter.length > 0) {
            reporter = json.reporter;
        } else {
            reporter = null;
        }
    }

    function onFieldsChanged() {
        var startDate = issueContainerUtils.getStartDateField().val();
        var endDate = issueContainerUtils.getEndDateField().val();
        var requestedMinutes = issueContainerUtils.getEndTimeMinutesField().val();
        var periodType;
        // TODO get rid of try-catch and prevent errors caused by missing custom fields VM-503
        try {
        	periodType = issueContainerUtils.getSelectedPeriodType();
        } catch(error) {
        	console.error(error);
        	return;
        }

        if (periodType === 'HOURLY') {
            startDate = issueContainerUtils.getStartTimeField().val();
        }

        if (periodType === 'HOURLY' && !requestedMinutes) {
            requestedMinutes = 0;
        } else if (periodType === 'HALF_DAILY' && !requestedHalfDay) {
            requestedHalfDay = 'AM';
        }

        var params = {
            vacationTypeId: vacationTypeId,
            selectedStartDate: startDate,
            selectedEndDate: endDate,
            periodType: periodType,
            requestedMinutes: parseInt(requestedMinutes, 10),
            requestedHalfDay: requestedHalfDay,
            reporter: issueContainerUtils.getReporterName()
        };
        if (issueId) {
            params.issueId = issueId;
        }

        getVacationInfo(params);
    }
    
    function getVacationInfo(params) {
        container.find('.vm_chart_spin').spin();
        jQuery.ajax({
            type : "POST",
            url : AJS.contextPath() + "/rest/vacation-manager/1.0/user-vacation/vacation-info",
            data : JSON.stringify(params),
            dataType : "json",
            contentType : 'application/json; charset=utf-8',
            async : true
        }).done(function(data) {
            container.find(".vm_chart_spin").spinStop();
            if (data.errors && data.errors.length) {
                errors = data.errors;
            } else {
                parseValuesFromJson(data);
                errors = null;
            }
            updateChart();
        });
    }

    function showWarningMessage(text) {
        if (text == undefined || text == null || text.length == 0) {
            hideWarningMessage();
            return;
        }

        vmMessages.html(text);
        vmMessages.show();
    }

    function hideWarningMessage() {
        vmMessages.hide();
    }
    
    function getSelectedHalfOfDay(value) {
    	if (value === VM$("[name=vmHalfDayAmOptionId]").val()) {
    		return "AM";
    	} else if (value === VM$("[name=vmHalfDayPmOptionId]").val()) {
    		return "PM";
    	}
    	throw new Error('Unsupported half day type option \'' + value + '\'');
    }

    init();
}
