function VacationDaysPieChart(params) {

    var chartColors = {
        availableDays: '#67ab49',
        remainingDays: '#67ab49',
        selectedDays: '#59afe1',
        reservedDays: '#f79232',
        usedDays: '#d04437'
    };

    var defaultChartOptions = {
        chartArea: {
            left: 10,
            top: 10,
            width: '100%',
            height: 180
        },
        legend : {
            textStyle: {
                color: '#707070'
            }
        },
        colors: [],
        pieSliceText : 'value'
    };

    params.chartOptions = AJS.$.extend(true, {}, defaultChartOptions, params.chartOptions);

    return {
        onLoad: onLoad,
        draw: draw
    };

    function onLoad(callback) {
        google.charts.setOnLoadCallback(callback);
    }

    function draw(data) {
        var chartData = new google.visualization.DataTable();
        chartData.addColumn('string', 'Time');
        chartData.addColumn('number', 'Amount');
        chartData.addColumn({
            type : 'string',
            role : 'tooltip'
        });

        var chartRows = [];
        var colors = [];
        if (data.availableDays) {
            chartRows.push(prepareAvailableDaysRow(data));
            colors.push(chartColors.availableDays);
        }
        if (data.remainingDays) {
            chartRows.push(prepareRemainingDaysRow(data));
            colors.push(chartColors.remainingDays);
        }
        if (data.usedDays) {
            chartRows.push(prepareUsedDaysRow(data));
            colors.push(chartColors.usedDays);
        }
        if (data.selectedDays) {
            chartRows.push(prepareSelectedDaysRow(data));
            colors.push(chartColors.selectedDays);
        }
        if (data.reservedDays) {
            chartRows.push(prepareReservedDaysRow(data));
            colors.push(chartColors.reservedDays);
        }

        chartData.addRows(chartRows);

        var chart = new google.visualization.PieChart(AJS.$(params.chartSelector)[0]);
        params.chartOptions.colors = colors;
        chart.draw(chartData, params.chartOptions);
    }

    function prepareAvailableDaysRow(data) {
        var label = AJS.I18n.getText('vacationmanager.vacation.customfield.statistics.available.days');
        var availableDaysLabel = AJS.I18n.getText('vacationmanager.vacation.customfield.statistics.available.days.description', data.availableDays);
        var leftoverDaysLabel = (data.leftoverDays > 0) ?
            ' ' + AJS.I18n.getText('vacationmanager.vacation.customfield.statistics.leftover.days.description', data.leftoverDays) : '';

        return [label, data.availableDays, availableDaysLabel + leftoverDaysLabel];
    }

    function prepareRemainingDaysRow(data) {
        var label = AJS.I18n.getText('vacationmanager.vacation.customfield.statistics.remaining.days');
        var remainingDaysLabel = AJS.I18n.getText('vacationmanager.vacation.customfield.statistics.remaining.days.description', data.remainingDays);
        var leftoverDaysLabel = (data.leftoverDays > 0) ?
            ' ' + AJS.I18n.getText('vacationmanager.vacation.customfield.statistics.leftover.days.description', data.leftoverDays) : '';

        return [label, data.remainingDays, remainingDaysLabel + leftoverDaysLabel];
    }

    function prepareSelectedDaysRow(data) {
        var label = AJS.I18n.getText('vacationmanager.vacation.customfield.statistics.chosen.days');
        var tooltipLabel = AJS.I18n.getText('vacationmanager.vacation.customfield.statistics.chosen.days.description', data.selectedDays);

        return [label, data.selectedDays, tooltipLabel];
    }

    function prepareReservedDaysRow(data) {
        return [AJS.I18n.getText('vacationmanager.vacation.customfield.statistics.reserved.days'),
            data.reservedDays,
            AJS.I18n.getText('vacationmanager.vacation.customfield.statistics.reserved.days.description', data.reservedDays)];
    }

    function prepareUsedDaysRow(data) {
        var label = AJS.I18n.getText('vacationmanager.vacation.customfield.statistics.used.days');
        var tooltipLabel = AJS.I18n.getText('vacationmanager.vacation.customfield.statistics.used.days.description', data.usedDays);

        return [label, data.usedDays, tooltipLabel];
    }

}